@@ -104,7 +104,9 @@ void add_extra_lib(char *);
 #undef X86_64
 #undef ARM
 #undef ARM64
+#undef MIPS
 #undef SPARC64
+#undef MIPS64
 
 #define UNKNOWN 0
 #define X86     1
@@ -119,6 +121,7 @@ void add_extra_lib(char *);
 #define ARM64   10
 #define MIPS    11
 #define SPARC64 12
+#define MIPS64  13
 
 #define TARGET_X86    "TARGET=X86"
 #define TARGET_ALPHA  "TARGET=ALPHA"
@@ -131,6 +134,7 @@ void add_extra_lib(char *);
 #define TARGET_ARM    "TARGET=ARM"
 #define TARGET_ARM64  "TARGET=ARM64"
 #define TARGET_MIPS   "TARGET=MIPS"
+#define TARGET_MIPS64 "TARGET=MIPS64"
 #define TARGET_SPARC64 "TARGET=SPARC64"
 
 #define TARGET_CFLAGS_X86    "TARGET_CFLAGS=-D_FILE_OFFSET_BITS=64"
@@ -152,6 +156,7 @@ void add_extra_lib(char *);
 #define TARGET_CFLAGS_MIPS            "TARGET_CFLAGS=-D_FILE_OFFSET_BITS=64"
 #define TARGET_CFLAGS_MIPS_ON_X86     "TARGET_CFLAGS=-D_FILE_OFFSET_BITS=64"
 #define TARGET_CFLAGS_MIPS_ON_X86_64  "TARGET_CFLAGS=-m32 -D_FILE_OFFSET_BITS=64"
+#define TARGET_CFLAGS_MIPS64          "TARGET_CFLAGS="
 #define TARGET_CFLAGS_SPARC64         "TARGET_CFLAGS="
 
 #define GDB_TARGET_DEFAULT        "GDB_CONF_FLAGS="
@@ -380,7 +385,11 @@ get_current_configuration(struct supported_gdb_version *sp)
         target_data.target = ARM64;
 #endif
 #ifdef __mips__
-        target_data.target = MIPS;
+#ifndef __mips64
+	target_data.target = MIPS;
+#else
+	target_data.target = MIPS64;
+#endif
 #endif
 #ifdef __sparc_v9__
 	target_data.target = SPARC64;
@@ -474,6 +483,10 @@ get_current_configuration(struct supported_gdb_version *sp)
 				arch_mismatch(sp);
 		}
 
+		if ((target_data.initial_gdb_target == MIPS64) &&
+		    (target_data.target != MIPS64))
+			arch_mismatch(sp);
+
 		if ((target_data.initial_gdb_target == X86) &&
 		    (target_data.target != X86)) {
 			if (target_data.target == X86_64) 
@@ -631,6 +644,9 @@ show_configuration(void)
 	case MIPS:
 		printf("TARGET: MIPS\n");
 		break;
+	case MIPS64:
+		printf("TARGET: MIPS64\n");
+		break;
 	case SPARC64:
 		printf("TARGET: SPARC64\n");
 		break;
@@ -742,7 +758,11 @@ build_configure(struct supported_gdb_version *sp)
 			gdb_conf_flags = GDB_TARGET_MIPS_ON_X86_64;
 		} else
                         target_CFLAGS = TARGET_CFLAGS_MIPS;
-                break;
+		break;
+	case MIPS64:
+		target = TARGET_MIPS64;
+		target_CFLAGS = TARGET_CFLAGS_MIPS64;
+		break;
 	case SPARC64:
 		target = TARGET_SPARC64;
 		target_CFLAGS = TARGET_CFLAGS_SPARC64;
@@ -1344,7 +1364,7 @@ make_spec_file(struct supported_gdb_version *sp)
 	printf("Vendor: Red Hat, Inc.\n");
 	printf("Packager: Dave Anderson <anderson@redhat.com>\n");
 	printf("ExclusiveOS: Linux\n");
-	printf("ExclusiveArch: %%{ix86} alpha ia64 ppc ppc64 ppc64pseries ppc64iseries x86_64 s390 s390x arm aarch64 ppc64le mips mipsel sparc64\n");
+	printf("ExclusiveArch: %%{ix86} alpha ia64 ppc ppc64 ppc64pseries ppc64iseries x86_64 s390 s390x arm aarch64 ppc64le mips mipsel mips64el sparc64\n");
 	printf("Buildroot: %%{_tmppath}/%%{name}-root\n");
 	printf("BuildRequires: ncurses-devel zlib-devel bison\n");
 	printf("Requires: binutils\n");
@@ -1571,6 +1591,8 @@ set_initial_target(struct supported_gdb_version *sp)
 		target_data.initial_gdb_target = ARM64;
 	else if (strncmp(buf, "ARM", strlen("ARM")) == 0)
 		target_data.initial_gdb_target = ARM;
+	else if (strncmp(buf, "MIPS64", strlen("MIPS64")) == 0)
+		target_data.initial_gdb_target = MIPS64;
 	else if (strncmp(buf, "MIPS", strlen("MIPS")) == 0)
 		target_data.initial_gdb_target = MIPS;
 	else if (strncmp(buf, "SPARC64", strlen("SPARC64")) == 0)
@@ -1593,6 +1615,7 @@ target_to_name(int target)
 	case ARM:    return("ARM"); 
 	case ARM64:  return("ARM64");
 	case MIPS:   return("MIPS");
+	case MIPS64: return("MIPS64");
 	case SPARC64: return("SPARC64");
 	}
 
@@ -1652,6 +1675,10 @@ name_to_target(char *name)
                 return MIPS;
         else if (strncmp(name, "MIPS", strlen("MIPS")) == 0)
                 return MIPS;
+	else if (strncmp(name, "mips64", strlen("mips64")) == 0)
+		return MIPS64;
+	else if (strncmp(name, "MIPS64", strlen("MIPS64")) == 0)
+		return MIPS64;
 	else if (strncmp(name, "sparc64", strlen("sparc64")) == 0)
 		return SPARC64;
 
@@ -1704,18 +1731,22 @@ get_extra_flags(char *filename, char *initial)
  *  a CFLAGS.extra file and an LDFLAGS.extra file.
 
  *  For lzo: 
+ *    - enter -DLZO in the CFLAGS.extra file
+ *    - enter -llzo2 in the LDFLAGS.extra file
+ *
+ *  For snappy:
  *    - enter -DSNAPPY in the CFLAGS.extra file
  *    - enter -lsnappy in the LDFLAGS.extra file
  *
- *  For snappy:
- *    - enter -DLZO in the CFLAGS.extra file
- *    - enter -llzo2 in the LDFLAGS.extra file.
+ *  For valgrind:
+ *    - enter -DVALGRIND in the CFLAGS.extra file
  */
 void
 add_extra_lib(char *option)
 {
 	int lzo, add_DLZO, add_llzo2; 
 	int snappy, add_DSNAPPY, add_lsnappy;
+	int valgrind, add_DVALGRIND;
 	char *cflags, *ldflags;
 	FILE *fp_cflags, *fp_ldflags;
 	char *mode;
@@ -1723,6 +1754,7 @@ add_extra_lib(char *option)
 
 	lzo = add_DLZO = add_llzo2 = 0;
 	snappy = add_DSNAPPY = add_lsnappy = 0;
+	valgrind = add_DVALGRIND = 0;
 
 	ldflags = get_extra_flags("LDFLAGS.extra", NULL);
 	cflags = get_extra_flags("CFLAGS.extra", NULL);
@@ -1743,12 +1775,24 @@ add_extra_lib(char *option)
 			add_lsnappy++;
 	}
 
+	if (strcmp(option, "valgrind") == 0) {
+		valgrind++;
+		if (!cflags || !strstr(cflags, "-DVALGRIND"))
+			add_DVALGRIND++;
+	}
+
 	if ((lzo || snappy) &&
 	    file_exists("diskdump.o") && (unlink("diskdump.o") < 0)) {
 		perror("diskdump.o");
 		return;
 	} 
 
+	if (valgrind &&
+	    file_exists("tools.o") && (unlink("tools.o") < 0)) {
+		perror("tools.o");
+		return;
+	}
+
 	mode = file_exists("CFLAGS.extra") ? "r+" : "w+";
 	if ((fp_cflags = fopen("CFLAGS.extra", mode)) == NULL) {
 		perror("CFLAGS.extra");
@@ -1762,13 +1806,15 @@ add_extra_lib(char *option)
 		return;
 	}
 
-	if (add_DLZO || add_DSNAPPY) {
+	if (add_DLZO || add_DSNAPPY || add_DVALGRIND) {
 		while (fgets(inbuf, 512, fp_cflags))
 			;
 		if (add_DLZO)
 			fputs("-DLZO\n", fp_cflags);
 		if (add_DSNAPPY)
 			fputs("-DSNAPPY\n", fp_cflags);
+		if (add_DVALGRIND)
+			fputs("-DVALGRIND\n", fp_cflags);
 	}
 
 	if (add_llzo2 || add_lsnappy) {
